@@ -1,13 +1,12 @@
 #! perl
 
 # Copyright (C) 2001-2008, The Perl Foundation.
-# $Id: Configure.pl 30645 2008-08-30 01:12:20Z jkeenan $
+# $Id: /mirror/trunk/Configure.pl 32213 2008-10-26T17:44:39.948814Z jkeenan  $
 
 use 5.008;
 use strict;
 use warnings;
 use lib 'lib';
-use Data::Dumper;$Data::Dumper::Indent = 1;
 
 use Parrot::Configure;
 use Parrot::Configure::Options qw( process_options );
@@ -42,7 +41,6 @@ my ($args, $steps_list_ref) = process_options(
         argv => [@ARGV],
     }
 );
-#print STDERR Dumper ($args, $steps_list_ref);
 exit(1) unless defined $args;
 
 my $opttest = Parrot::Configure::Options::Test->new($args);
@@ -649,7 +647,7 @@ or
 
 =head1 SEE ALSO
 
-F<config/init/data.pl>, F<lib/Parrot/Configure.pm>,
+F<lib/Parrot/Configure.pm>,
 F<lib/Parrot/Configure/Step.pm>, F<docs/configuration.pod>
 
 =cut
