@@ -1,9 +1,9 @@
 /*
- * $Id: imc.h 32013 2008-10-18 16:41:30Z Whiteknight $
+ * $Id: /mirror/trunk/compilers/imcc/imc.h 32689 2008-11-15T06:28:00.868249Z chromatic  $
  * Copyright (C) 2002-2008, The Perl Foundation.
  */
 
-/* $Id: imc.h 32013 2008-10-18 16:41:30Z Whiteknight $ */
+/* $Id: /mirror/trunk/compilers/imcc/imc.h 32689 2008-11-15T06:28:00.868249Z chromatic  $ */
 
 #ifndef PARROT_IMCC_IMC_H_GUARD
 #define PARROT_IMCC_IMC_H_GUARD
@@ -14,11 +14,11 @@
 #ifdef PARROT_HAS_HEADER_SYSEXITS
 #  include <sysexits.h>
 #else
-#  define EX_DATAERR 1
-#  define EX_SOFTWARE 1
-#  define EX_NOINPUT 1
-#  define EX_IOERR 1
-#  define EX_USAGE 1
+#  define EX_DATAERR     1
+#  define EX_SOFTWARE    1
+#  define EX_NOINPUT     1
+#  define EX_IOERR       1
+#  define EX_USAGE       1
 #  define EX_UNAVAILABLE 1
 #endif  /* PARROT_HAS_HEADER_SYSEXITS */
 
@@ -398,11 +398,6 @@ SymReg* get_pasm_reg(PARROT_INTERP, ARGIN(const char *name))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 /* HEADERIZER END: compilers/imcc/pcc.c */
 
-/* pragmas avialable: */
-typedef enum {
-  PR_N_OPERATORS = 0x01
-} _imc_pragmas;
-
 #define PARROT_MAX_RECOVER_ERRORS 40
 /* this is the number of times parrot will try to retry while compiling. */
 
@@ -417,18 +412,15 @@ typedef enum _enum_opt {
 
 struct nodeType_t;
 
-/*
- * see also imcc/imcc.l struct macro_frame_t
- */
+/* see also imcc/imcc.l struct macro_frame_t */
 struct parser_state_t {
     struct parser_state_t *next;
-    Interp *interp;
-    const char *file;
-    int file_needs_free; /* If *file is malloced, we need to free it */
-    FILE *handle;
-    int line;
-    int pasm_file;       /* pasm_file mode of this frame */
-    int pragmas;         /* n_operators ... */
+    Interp                *interp;
+    const char            *file;
+    FILE                  *handle;
+    int                    file_needs_free; /* is *file malloced? */
+    int                    line;
+    int                    pasm_file;       /* pasm_file mode of this frame */
 };
 
 typedef enum _AsmState {
@@ -446,74 +438,140 @@ typedef enum _imcc_reg_allocator_t {
 PARROT_API void IMCC_push_parser_state(PARROT_INTERP);
 PARROT_API void IMCC_pop_parser_state(PARROT_INTERP, void *yyscanner);
 
+/* globals store the state between individual e_pbc_emit calls */
+typedef struct subs_t {
+    IMC_Unit      *unit;
+    struct subs_t *prev;
+    struct subs_t *next;
+    SymHash        fixup;              /* currently set_p_pc sub names only */
+    int            ins_line;           /* line number for debug */
+    int            n_basic_blocks;     /* block count */
+    int            pmc_const;          /* index in const table */
+    size_t         size;               /* code size in ops */
+} subs_t;
+
+/* subs are kept per code segment */
+typedef struct code_segment_t {
+    PackFile_ByteCode     *seg;           /* bytecode segment */
+    PackFile_Segment      *jit_info;      /* bblocks, register usage */
+    subs_t                *subs;          /* current sub data */
+    subs_t                *first;         /* first sub of code segment */
+    struct code_segment_t *prev;          /* previous code segment */
+    struct code_segment_t *next;          /* next code segment */
+    SymHash                key_consts;    /* this seg's cached key constants */
+    int                    pic_idx;       /* next index of PIC */
+} code_segment_t;
+
+typedef struct _imcc_globals_t {
+    code_segment_t *cs;           /* current code segment */
+    code_segment_t *first;        /* first code segment   */
+    int             inter_seg_n;
+} imcc_globals;
+
 typedef struct _imc_info_t {
-    struct _imc_info_t *prev;
-    IMC_Unit * imc_units;
-    IMC_Unit * last_unit;
-    IMC_Unit * cur_unit;
-    int n_comp_units;
-    int imcc_warn;
-    int verbose;
-    int debug;
-    int IMCC_DEBUG;
-    int gc_off;
-    int write_pbc;
-    int allocator;
-    SymReg* sr_return;
-    AsmState asm_state;
-    int optimizer_level;
-    int dont_optimize;
-    int has_compile;
-    int allocated;
-    SymHash ghash;
-    SymReg  *  cur_namespace;
-    struct nodeType_t *top_node;
+    void                  *yyscanner;
+    struct _imc_info_t    *prev;
+    IMC_Unit              *imc_units;
+    IMC_Unit              *last_unit;
+    IMC_Unit              *cur_unit;
+    SymReg                *sr_return;
+    SymReg                *cur_namespace;
+    struct nodeType_t     *top_node;
     struct parser_state_t *state;
-    jmp_buf jump_buf;               /* The jump for error  handling */
-    int error_code;                 /* The Error code. */
-    STRING * error_message;         /* The Error message */
+    STRING                *error_message;   /* The Error message */
 
     /* some values that were global... */
-    int line;                   /* current line number */
-    int cur_pmc_type;
-    SymReg *cur_call;
-    SymReg *cur_obj;
-    const char *adv_named_id;
+    SymReg               *cur_call;
+    SymReg               *cur_obj;
+    const char           *adv_named_id;
 
     /* Lex globals */
-    int in_pod;
-    char *heredoc_end;
-    char *heredoc_content;
-    char *cur_macro_name;
+    char                 *heredoc_end;
+    char                 *heredoc_content;
+    char                 *cur_macro_name;
 
-    int expect_pasm;
     struct macro_frame_t *frames;
+    imcc_globals         *globals;
 
-    char *macro_buffer;
-    Hash *macros;
+    char                 *macro_buffer;
+    Hash                 *macros;
+    PackFile_Debug       *debug_seg;
+    opcode_t             *pc;
 
-    /*
-     * these are used for constructing one INS
-     */
+    /* these are used for constructing one INS */
 #define IMCC_MAX_STATIC_REGS 100
-    SymReg *regs[IMCC_MAX_STATIC_REGS];
-    int nkeys;
-    SymReg *keys[IMCC_MAX_FIX_REGS]; /* TODO key overflow check */
-    int keyvec;
-    int cnr;
-    int nargs;
-    int in_slice;
-    void *yyscanner;
+    SymReg               *regs[IMCC_MAX_STATIC_REGS];
+    SymReg               *keys[IMCC_MAX_FIX_REGS]; /* TODO key overflow check */
+    AsmState              asm_state;
+    SymHash               ghash;
+    jmp_buf               jump_buf;        /* The jump for error  handling */
+    int                   IMCC_DEBUG;
+    int                   allocated;
+    int                   allocator;
+    int                   cnr;
+    int                   cur_pmc_type;
+    int                   debug;
+    int                   dont_optimize;
+    int                   emitter;
+    int                   error_code;      /* The Error code. */
+    int                   expect_pasm;
+    int                   gc_off;
+    int                   has_compile;
+    int                   imcc_warn;
+    int                   in_pod;
+    int                   in_slice;
+    int                   ins_line;
+    int                   keyvec;
+    int                   line;                   /* current line number */
+    int                   optimizer_level;
+    int                   nargs;
+    int                   n_comp_units;
+    int                   nkeys;
+    int                   compiler_state;         /* see PBC_* flags */
+    int                   verbose;
+    int                   write_pbc;
+    opcode_t              npc;
 } imc_info_t;
 
 #define IMCC_INFO(i) (((Parrot_Interp)(i))->imc_info)
 
-#define IMC_TRACE 0
+#define IMC_TRACE      0
 #define IMC_TRACE_HIGH 0
 
+/* main.c */
+#define PBC_LOAD        (1 << 0)
+#define PBC_RUN         (1 << 1)
+#define PBC_WRITE       (1 << 2)
+#define PBC_PRE_PROCESS (1 << 3)
+#define PBC_PASM_FILE   (1 << 4)
+#define PBC_RUN_FILE    (1 << 5)
+
+#define COMPILER_STATE(i) IMCC_INFO(i)->compiler_state
+
+#define STATE_LOAD_PBC(i)      (COMPILER_STATE(i) & PBC_LOAD)
+#define STATE_RUN_PBC(i)       (COMPILER_STATE(i) & PBC_RUN)
+#define STATE_WRITE_PBC(i)     (COMPILER_STATE(i) & PBC_WRITE)
+#define STATE_PRE_PROCESS(i)   (COMPILER_STATE(i) & PBC_PRE_PROCESS)
+#define STATE_PASM_FILE(i)     (COMPILER_STATE(i) & PBC_PASM_FILE)
+#define STATE_RUN_FROM_FILE(i) (COMPILER_STATE(i) & (PBC_RUN_FILE | PBC_RUN))
+
+#define SET_STATE_LOAD_PBC(i)      (COMPILER_STATE(i) |= PBC_LOAD)
+#define SET_STATE_RUN_PBC(i)       (COMPILER_STATE(i) |= PBC_RUN)
+#define SET_STATE_WRITE_PBC(i)     (COMPILER_STATE(i) |= PBC_WRITE)
+#define SET_STATE_PRE_PROCESS(i)   (COMPILER_STATE(i) |= PBC_PRE_PROCESS)
+#define SET_STATE_PASM_FILE(i)     (COMPILER_STATE(i) |= PBC_PASM_FILE)
+#define SET_STATE_RUN_FROM_FILE(i) (COMPILER_STATE(i) |= PBC_RUN_FILE)
+
+#define UNSET_STATE_LOAD_PBC(i)      (COMPILER_STATE(i) &= ~PBC_LOAD)
+#define UNSET_STATE_RUN_PBC(i)       (COMPILER_STATE(i) &= ~PBC_RUN)
+#define UNSET_STATE_WRITE_PBC(i)     (COMPILER_STATE(i) &= ~PBC_WRITE)
+#define UNSET_STATE_PRE_PROCESS(i)   (COMPILER_STATE(i) &= ~PBC_PRE_PROCESS)
+#define UNSET_STATE_PASM_FILE(i)     (COMPILER_STATE(i) &= ~PBC_PASM_FILE)
+#define UNSET_STATE_RUN_FROM_FILE(i) (COMPILER_STATE(i) &= ~PBC_RUN_FILE)
+
 /* imclexer.c */
-PARROT_API FILE* imc_yyin_set(FILE* new_yyin, void *yyscanner);
-PARROT_API FILE* imc_yyin_get(void *yyscanner);
+PARROT_API FILE * imc_yyin_set(FILE *new_yyin, void *yyscanner);
+PARROT_API FILE * imc_yyin_get(void *yyscanner);
 
 
 #endif /* PARROT_IMCC_IMC_H_GUARD */
