@@ -1,4 +1,4 @@
-/* $Id: instructions.c 31600 2008-10-03 23:10:30Z Whiteknight $ */
+/* $Id: /mirror/trunk/compilers/imcc/instructions.c 32537 2008-11-11T04:38:30.960026Z chromatic  $ */
 
 /*
  * Copyright (C) 2002-2008, The Perl Foundation.
@@ -35,14 +35,6 @@ These functions operate over this array and its contents.
 
 */
 
-/* Global variables , forward def */
-
-#if 0
-static Instruction * last_ins;
-
-int n_comp_units;
-#endif
-
 /* HEADERIZER HFILE: compilers/imcc/instructions.h */
 
 /* HEADERIZER BEGIN: static */
@@ -80,8 +72,6 @@ static const Emitter emitters[] = {
      e_pbc_close},
 };
 
-static int emitter;     /* XXX */
-
 /*
 
 =item C<Instruction * _mk_instruction>
@@ -847,11 +837,11 @@ PARROT_API
 int
 emit_open(PARROT_INTERP, int type, ARGIN_NULLOK(void *param))
 {
-    emitter                          = type;
+    IMCC_INFO(interp)->emitter       = type;
     IMCC_INFO(interp)->has_compile   = 0;
     IMCC_INFO(interp)->dont_optimize = 0;
 
-    return (emitters[emitter]).open(interp, param);
+    return (emitters[IMCC_INFO(interp)->emitter]).open(interp, param);
 }
 
 /*
@@ -869,7 +859,8 @@ PARROT_API
 int
 emit_flush(PARROT_INTERP, ARGIN_NULLOK(void *param), ARGIN(IMC_Unit *unit))
 {
-    Instruction * ins;
+    Instruction *ins;
+    int          emitter = IMCC_INFO(interp)->emitter;
 
     if (emitters[emitter].new_sub)
         (emitters[emitter]).new_sub(interp, param, unit);
@@ -899,7 +890,7 @@ PARROT_API
 int
 emit_close(PARROT_INTERP, ARGIN_NULLOK(void *param))
 {
-    return (emitters[emitter]).close(interp, param);
+    return (emitters[IMCC_INFO(interp)->emitter]).close(interp, param);
 }
 
 /*
