@@ -1,5 +1,5 @@
 /*
- * $Id: pbc.h 27452 2008-05-12 02:15:12Z petdance $
+ * $Id: /mirror/trunk/compilers/imcc/pbc.h 32493 2008-11-09T07:23:27.381729Z chromatic  $
  * Copyright (C) 2002-2007, The Perl Foundation.
  */
 
@@ -24,7 +24,8 @@ int e_pbc_end_sub(PARROT_INTERP, SHIM(void *param), ARGIN(IMC_Unit *unit))
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
-int e_pbc_new_sub(SHIM_INTERP, SHIM(void *param), ARGIN(IMC_Unit *unit))
+int e_pbc_new_sub(PARROT_INTERP, SHIM(void *param), ARGIN(IMC_Unit *unit))
+        __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
 int e_pbc_open(PARROT_INTERP, SHIM(void *param))
