@@ -1,5 +1,5 @@
 /*
- * $Id: main.c 32029 2008-10-18 23:37:06Z kjs $
+ * $Id: /mirror/trunk/compilers/pirc/new/main.c 32766 2008-11-16T23:38:46.293841Z kjs  $
  * Copyright (C) 2007-2008, The Perl Foundation.
  */
 
@@ -13,23 +13,38 @@
 #include "piremit.h"
 #include "piryy.h"
 #include "pirlexer.h"
+#include "pirheredoc.h"
+#include "pirregalloc.h"
+
+/* global variable to set parser in debug mode.
+ * It is not clear to me whether the global can be replaced
+ * by a parser-specific flag.
+ */
+#ifdef YYDEBUG
+
+extern int yypirdebug = 0;
+
+#endif
+
 
 
 /* XXX use pthreads library to test thread safety.
    does not work currently on windows.
    The check for _MSC_VER is not correct but works for me.
 */
+
+/*
 #ifndef _MSC_VER
 #  define TEST_THREAD_SAFETY
 #endif
-
+*/
 
 #ifdef TEST_THREAD_SAFETY
 #  include <pthread.h>
 #  define NUM_THREADS 1
 #endif
 
-void * parse_file(void *a);
+void * process_file(void *a);
 
 /*
 
@@ -50,15 +65,24 @@ static void
 print_help(char const * const program_name)
 {
     fprintf(stderr, "Usage: %s [options] <file>\n", program_name);
-    fprintf(stderr, "Options:\n\n");
-    /*fprintf(stderr, "  -E        pre-process\n"); */
-    fprintf(stderr, "  -d        show debug messages of parser\n");
-    fprintf(stderr, "  -h        show this help message\n");
-    fprintf(stderr, "  -W        show warning messages\n");
-    fprintf(stderr, "  -S        do not perform strength reduction\n");
-    fprintf(stderr, "  -v        verbose mode\n");
-    fprintf(stderr, "  -o <file> write output to the specified file. "
-                    "Currently only works in combination with '-E' option\n");
+    fprintf(stderr, "Options:\n\n"
+    "  -E        run heredoc and macro preprocessors only\n"
+    "  -d        show debug messages of parser\n"
+    "  -h        show this help message\n"
+    "  -H        heredoc preprocessing only\n"
+    "  -m <size> specify initial macro buffer size; default is 4096 bytes\n"
+    "  -n        no output, only print 'ok' if successful\n"
+    "  -o <file> write output to the specified file. "
+    "Currently only works in combination with '-E' option\n"
+    "  -p        pasm output\n"
+    "  -r        activate the register allocator for improved register usage\n"
+    "  -S        do not perform strength reduction\n"
+    "  -v        verbose mode\n"
+    "  -W        show warning messages\n"
+#ifdef YYDEBUG
+    "  -y        debug bison-generated parser\n"
+#endif
+    );
 }
 
 
@@ -110,22 +134,19 @@ typedef struct parser_args {
 
 This will be the proper declaration after testing for thread-safety:
 
-void parse_file(int flexdebug, FILE *infile, char * const filename)
+void parse_file(int flexdebug, FILE *infile, char * const filename, int flags)
 
 */
-void *
-parse_file(void *a) {
+
+void init_scanner_state(yyscan_t yyscanner);
+
+void
+parse_file(int flexdebug, FILE *infile, char * const filename, int flags, int thr_id,
+           unsigned macro_size)
+{
     yyscan_t     yyscanner;
     lexer_state *lexer     = NULL;
 
-    /* unpack the arguments from the structure parser_args */
-    parser_args *args      = (parser_args *)a;
-    int          flexdebug = args->flexdebug;
-    FILE        *infile    = args->infile;
-    char        *filename  = args->filename;
-    int          thr_id    = args->thr_id;
-    int          flags     = args->flags;
-
     /* create a yyscan_t object */
     yypirlex_init(&yyscanner);
     /* set debug flag */
@@ -134,6 +155,15 @@ parse_file(void *a) {
     yypirset_in(infile, yyscanner);
     /* set the extra parameter in the yyscan_t structure */
     lexer = new_lexer(filename, flags);
+    lexer->macro_size = macro_size;
+
+    /* initialize the scanner state */
+    init_scanner_state(yyscanner);
+
+    if (strstr(filename, ".pasm")) { /* PASM mode */
+        SET_FLAG(lexer->flags, LEXER_FLAG_PASMFILE);
+    }
+
     yypirset_extra(lexer, yyscanner);
     /* and store the yyscanner in the lexer, so they're close buddies */
     lexer->yyscanner = yyscanner;
@@ -144,11 +174,26 @@ parse_file(void *a) {
         char outfile[20];
         sprintf(outfile, "output_thr_%d", thr_id);
         lexer->outfile = open_file(outfile, "w");
-        if (lexer->outfile == NULL)
+        if (lexer->outfile == NULL) {
             fprintf(stderr, "Failed to open file %s\n", outfile);
+            lexer->outfile = stdout;
+        }
+
+        /* if register allocation was requested, do that now */
+        if (TEST_FLAG(lexer->flags, LEXER_FLAG_REGALLOC))
+            linear_scan_register_allocation(lexer->lsr);
+
+        if (TEST_FLAG(lexer->flags, LEXER_FLAG_NOOUTPUT)) /* handy for testing the compiler */
+            fprintf(stdout, "ok\n");
+        else if (TEST_FLAG(lexer->flags, LEXER_FLAG_PREPROCESS))
+            emit_pir_subs(lexer);
+        else {
+            /*
+            fprintf(stderr, "Parse successful!\n");
+            */
+            print_subs(lexer);
+        }
 
-        fprintf(stderr, "Parse successful!\n");
-        print_subs(lexer);
         fclose(lexer->outfile);
 
         if (TEST_FLAG(lexer->flags, LEXER_FLAG_WARNINGS))
@@ -166,13 +211,34 @@ parse_file(void *a) {
     /* clean up after playing */
     release_resources(lexer);
     yypirlex_destroy(yyscanner);
+}
+
+/*
+
+temporary function for the thread-testing code.
+Unpack the arguments and invoke parse_file().
+
+*/
+void *
+process_file(void *a) {
+
+
+    /* unpack the arguments from the structure parser_args */
+    parser_args *args      = (parser_args *)a;
+    int          flexdebug = args->flexdebug;
+    FILE        *infile    = args->infile;
+    char        *filename  = args->filename;
+    int          thr_id    = args->thr_id;
+    int          flags     = args->flags;
+
+    parse_file(flexdebug, infile, filename, flags, thr_id, INIT_MACRO_SIZE);
 
     return NULL;
-
 }
 
 
 
+
 /*
 
 =item C<int
@@ -190,22 +256,22 @@ main(int argc, char *argv[]) {
     int                flags        = 0;
     char              *filename     = NULL;
     char              *outputfile   = NULL;
+    unsigned           macrosize    = INIT_MACRO_SIZE;
+
 
     /* skip program name */
     argc--;
     argv++;
 
 
+
     /* XXX very basic argument handling; I'm too lazy to check out
      * the standard funtion for that, right now. This is a TODO. */
     while (argc > 0 && argv[0][0] == '-') {
         switch (argv[0][1]) {
-            /* Only allow for debug flag if the generated parser supports it */
-#ifdef YYDEBUG
-            case 'd':
-                yydebug = 1;
+            case 'E':
+                SET_FLAG(flags, LEXER_FLAG_PREPROCESS);
                 break;
-#endif
             case 'f':
                 flexdebug = 1;
                 break;
@@ -213,6 +279,23 @@ main(int argc, char *argv[]) {
                 print_help(program_name);
                 exit(EXIT_SUCCESS); /* asking for help doesn't make you a failure */
                 /* break; */
+            case 'H':
+                SET_FLAG(flags, LEXER_FLAG_HEREDOCONLY);
+                break;
+            case 'm':
+                if (argc > 1) {
+                    argc--;
+                    argv++;
+                    macrosize = atoi(argv[0]);
+                }
+                else {
+                    fprintf(stderr, "Missing argument for option '-m'\n");
+                    exit(EXIT_FAILURE);
+                }
+                break;
+            case 'n':
+                SET_FLAG(flags, LEXER_FLAG_NOOUTPUT);
+                break;
             case 'o':
                 if (argc > 1) { /* there must be at least 2 more args,
                                          the output file, and an input */
@@ -228,8 +311,8 @@ main(int argc, char *argv[]) {
             case 'p':
                 SET_FLAG(flags, LEXER_FLAG_EMIT_PASM);
                 break;
-            case 'W':
-                SET_FLAG(flags, LEXER_FLAG_WARNINGS);
+            case 'r':
+                SET_FLAG(flags, LEXER_FLAG_REGALLOC);
                 break;
             case 'S':
                 SET_FLAG(flags, LEXER_FLAG_NOSTRENGTHREDUCTION);
@@ -237,6 +320,15 @@ main(int argc, char *argv[]) {
             case 'v':
                 SET_FLAG(flags, LEXER_FLAG_VERBOSE);
                 break;
+            case 'W':
+                SET_FLAG(flags, LEXER_FLAG_WARNINGS);
+                break;
+/* Only allow for debug flag if the generated parser supports it */
+#ifdef YYDEBUG
+            case 'y':
+                yypirdebug = 1;
+                break;
+#endif
             default:
                 fprintf(stderr, "Unknown option: '%c'\n", argv[0][1]);
                 exit(EXIT_FAILURE);
@@ -269,6 +361,7 @@ main(int argc, char *argv[]) {
             infile   = open_file(argv[0], "r");
             filename = argv[0];
         }
+
         if (infile == NULL) {
             fprintf(stderr, "Failed to open file '%s'\n", argv[0]);
             exit(EXIT_FAILURE);
@@ -280,7 +373,7 @@ main(int argc, char *argv[]) {
         args.thr_id    = i;
         args.flags     = flags;
 
-        pthread_create(&threads[i], NULL, parse_file, &args);
+        pthread_create(&threads[i], NULL, process_file, &args);
 
     }
 
@@ -290,31 +383,34 @@ main(int argc, char *argv[]) {
 }
 #else
 {
-    parser_args args;
-    FILE *infile = NULL;
+    /* non-thread testing code; this is the normal case */
+    FILE *file = NULL;
 
-    if (argc < 1) { /* no file specified, read from stdin */
-        infile   = stdin;
-        filename = NULL;
+    if (argc < 1) {
+        fprintf(stderr, "pirc: no input specified\n");
+        exit(EXIT_FAILURE);
     }
-    else {
-        /* done handling arguments, open the file */
-        infile   = open_file(argv[0], "r");
-        filename = argv[0];
+
+    /* if user requested only to process heredocs, send output to stdout and return. */
+    if (TEST_FLAG(flags, LEXER_FLAG_HEREDOCONLY)) {
+        process_heredocs(argv[0], stdout);
+        return 0;
     }
-    if (infile == NULL) {
+
+    file = fopen("heredoc.out", "w");
+    process_heredocs(argv[0], file);
+    fclose(file);
+    /* done handling arguments, open the file */
+    file     = open_file("heredoc.out", "r");
+    filename = argv[0];
+
+    if (file == NULL) {
         fprintf(stderr, "Failed to open file '%s'\n", argv[0]);
         exit(EXIT_FAILURE);
     }
 
-    /* pack all args for parse_file() */
-    args.flexdebug = flexdebug;
-    args.infile    = infile;
-    args.filename  = filename;
-    args.thr_id    = 0;
-    args.flags     = flags;
+    parse_file(flexdebug, file, filename, flags, 0, macrosize);
 
-    parse_file(&args);
 }
 #endif
 
@@ -326,7 +422,7 @@ main(int argc, char *argv[]) {
 /*
 
 =item C<int
-yyerror(yyscan_t yyscanner, lexer_state * const  lexer, char const * const message, ...)>
+yypirerror(yyscan_t yyscanner, lexer_state * const lexer, char const * const message, ...)>
 
 Default parse error handling routine, that is invoked when the bison-generated
 parser finds a syntax error.
