@@ -1,5 +1,5 @@
 /*
- * $Id: pircompiler.h 32029 2008-10-18 23:37:06Z kjs $
+ * $Id: /mirror/trunk/compilers/pirc/new/pircompiler.h 32383 2008-11-04T23:43:03.899228Z kjs  $
  * Copyright (C) 2007-2008, The Perl Foundation.
  */
 
@@ -8,7 +8,8 @@
 
 #include "pirsymbol.h"
 #include "pircompunit.h"
-
+#include "pirmacro.h"
+#include "pirregalloc.h"
 #include "parrot/parrot.h"
 #include "parrot/embed.h"
 
@@ -27,10 +28,15 @@ typedef struct cache {
 
 
 typedef enum lexer_flags {
-    LEXER_FLAG_WARNINGS            = 1 << 0,
-    LEXER_FLAG_NOSTRENGTHREDUCTION = 1 << 1,
-    LEXER_FLAG_VERBOSE             = 1 << 2,
-    LEXER_FLAG_EMIT_PASM           = 1 << 3
+    LEXER_FLAG_WARNINGS            = 1 << 0, /* print warnings */
+    LEXER_FLAG_NOSTRENGTHREDUCTION = 1 << 1, /* skip strength reduction */
+    LEXER_FLAG_VERBOSE             = 1 << 2, /* be verbose */
+    LEXER_FLAG_EMIT_PASM           = 1 << 3, /* emit runnable pasm code */
+    LEXER_FLAG_PREPROCESS          = 1 << 4, /* preprocess code only */
+    LEXER_FLAG_HEREDOCONLY         = 1 << 5, /* preprocess heredocs only */
+    LEXER_FLAG_NOOUTPUT            = 1 << 6, /* don't print anything on success, except 'ok' */
+    LEXER_FLAG_REGALLOC            = 1 << 7, /* use register allocation optimizer */
+    LEXER_FLAG_PASMFILE            = 1 << 8  /* the input is PASM, not PIR code */
 
 } lexer_flags;
 
@@ -39,7 +45,7 @@ typedef enum lexer_flags {
  * before creating a new node in the list of allocated_mem_ptrs structures,
  * but for small programs with few memory allocations, this means an overhead.
  * Likewise, if the number is too small, this means a lot of allocation of these
- * blocks, as they're full quickly. XXX maybe make this configurable?
+ * blocks, as they're full quickly. XXX maybe make this runtime configurable?
  *
  */
 #define NUM_MEM_ALLOCS_PER_BLOCK    512
@@ -72,6 +78,10 @@ typedef void * yyscan_t;
 
 #endif
 
+/* macros can store up to 4K characters, after which the buffer must be resized.
+ */
+#define INIT_MACRO_SIZE     4096
+
 
 /* store the "globals" of the lexer in a structure which is passed around.
  * as there's only 1 lexer_state, Size Doesn't Matter (really), so the large
@@ -94,6 +104,7 @@ typedef struct lexer_state {
     int            pir_reg_generator; /* for unique PIR register allocator, for temp. PIR regs. */
 
     unsigned       instr_counter;  /* to assign an index to each instruction */
+    unsigned       stmt_counter;   /* to count "logical" statements, even if multi-line. */
 
     Interp        *interp;         /* parrot interpreter */
 
@@ -112,9 +123,28 @@ typedef struct lexer_state {
                                     * lexer.
                                     */
 
+    /* macro stuff */
+    macro_table              *macros;        /* macro definitions; they are organized in
+                                              * C<macro_table>s, which are just symbol tables,
+                                              * representing scopes. Tables can be stacked,
+                                              * so it's easy to add a nested scope and pop it off
+                                              * after you're done with it.
+                                              */
+
+    int                       num_digits;    /* keep track of number of digits needed for id_gen */
+    int                       id_gen;        /* for generating unique identifiers */
+    int                       unique_id;     /* current unique id */
+    unsigned                  macro_size;    /* initial macro body buffr size */
+
+    struct yy_buffer_state   *buffer;  /* for saving buffer state when scanning a .macro_const */
+
+    /* register allocation */
+    lsr_allocator            *lsr;
 
 } lexer_state;
 
+/* accessor for current macro; always first on the list. */
+#define CURRENT_MACRO(X)    X->macros->definitions
 
 
 /* constructor for a lexer_state object */
@@ -133,8 +163,16 @@ void init_hashtable(lexer_state * const lexer, hashtable * const table, unsigned
 /* same trick as in parrot's memory system, for "automagic" casting */
 #define pir_mem_allocate_zeroed_typed(lxr, type) (type *)pir_mem_allocate_zeroed(lxr, sizeof (type))
 
+/* use pir_mem_allocate functions if you don't want to worry about freeing it; all memory
+ * allocated will be freed after the compilation. If you only need some memory temporarily
+ * and freeing can be done soon after (manually), then use parrot's mem_sys_allocate().
+ */
 void *pir_mem_allocate_zeroed(lexer_state * const lexer, size_t numbytes);
 
+void *pir_mem_allocate(lexer_state * const lexer, size_t numbytes);
+
+void pirwarning(lexer_state * const lexer, int lineno, char const * const message, ...);
+
 #endif /* PARROT_PIR_PIRCOMPILER_H_GUARD */
 
 /*
