@@ -1,7 +1,7 @@
 #! perl
 
 # Copyright (C) 2003-2006, The Perl Foundation.
-# $Id: kate_syntax.pl 27082 2008-04-21 20:08:48Z pmichaud $
+# $Id: /mirror/trunk/editor/kate_syntax.pl 32135 2008-10-22T16:38:09.702225Z bernhard  $
 
 # Create Kate syntax highlighting XML file for IMCC
 # Language:     Parrot IMCC
@@ -23,7 +23,7 @@ if ( $parrot eq '-h' ) {
 $parrot =~ s/\/$//;
 
 my @imcc_oplike = qw( .sym .arg prototyped non_prototyped .class
-    .endclass .param inc dec new defined addr global clone );
+    .endclass .param inc dec new defined global clone );
 
 my @imcc_spdirec = qw(.call .result .return .local .const .globalconst
     end goto if unless call branch jump jsr ret invoke invokecc throw
