@@ -1,7 +1,7 @@
 /* exceptions.h
  *  Copyright (C) 2001-2008, The Perl Foundation.
  *  SVN Info
- *     $Id: exceptions.h 31547 2008-10-01 14:53:32Z tene $
+ *     $Id: /mirror/trunk/include/parrot/exceptions.h 32253 2008-10-28T20:44:01.227920Z julianalbo  $
  *  Overview:
  *     define the internal interpreter exceptions
  *  Data Structure and Algorithms:
@@ -82,6 +82,7 @@ typedef enum {
         EXCEPTION_LIBRARY_ERROR,
         EXCEPTION_LIBRARY_NOT_LOADED,
         EXCEPTION_SYNTAX_ERROR,
+        EXCEPTION_MALFORMED_PACKFILE,
 
         CONTROL_RETURN,
         CONTROL_OK,
@@ -197,6 +198,16 @@ opcode_t * Parrot_ex_throw_from_op(PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_API
+PARROT_CAN_RETURN_NULL
+opcode_t * Parrot_ex_throw_from_op_args(PARROT_INTERP,
+    ARGIN_NULLOK(void *dest),
+    int ex_type,
+    ARGIN(const char *format),
+    ...)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(4);
+
 PARROT_DOES_NOT_RETURN
 void do_panic(
     NULLOK_INTERP,
