@@ -1,7 +1,7 @@
 /* interpreter.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: interpreter.h 32079 2008-10-21 20:35:06Z julianalbo $
+ *     $Id: /mirror/trunk/include/parrot/interpreter.h 32171 2008-10-23T17:21:19.756490Z julianalbo  $
  *  Overview:
  *     The interpreter API handles running the operations
  */
@@ -356,7 +356,7 @@ struct parrot_interp_t {
     INTVAL resume_flag;
     size_t resume_offset;
 
-    struct PackFile_ByteCode *code;           /* The code we are executing */
+    PackFile_ByteCode *code;                  /* The code we are executing */
     struct PackFile          *initial_pf;     /* first created PF  */
 
     struct _imc_info_t *imc_info;             /* imcc data */
