@@ -1,7 +1,7 @@
 /* multidispatch.h
  *  Copyright (C) 2003-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: multidispatch.h 31977 2008-10-15 17:09:09Z julianalbo $
+ *     $Id: /mirror/trunk/include/parrot/multidispatch.h 32388 2008-11-05T05:19:04.930668Z coke  $
  *  Overview:
  *     This is the API header for the mmd subsystem
  *  Data Structure and Algorithms:
@@ -76,16 +76,6 @@ PMC* Parrot_build_sig_object_from_varargs(PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-PARROT_API
-void Parrot_mmd_add_by_class(PARROT_INTERP,
-    INTVAL functype,
-    ARGIN(STRING *left_class),
-    ARGIN(STRING *right_class),
-    NULLOK(funcptr_t funcptr))
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(3)
-        __attribute__nonnull__(4);
-
 PARROT_API
 void Parrot_mmd_add_function(PARROT_INTERP,
     INTVAL func_nr,
@@ -155,10 +145,6 @@ void Parrot_mmd_multi_dispatch_from_c_args(PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
-PARROT_API
-void Parrot_mmd_rebuild_table(PARROT_INTERP, INTVAL type, INTVAL func_nr)
-        __attribute__nonnull__(1);
-
 PARROT_API
 void Parrot_mmd_register(PARROT_INTERP,
     INTVAL func_nr,
