@@ -1,7 +1,7 @@
 /* pic.h
  *  Copyright (C) 2005, The Perl Foundation.
  *  SVN Info
- *     $Id: pic.h 27452 2008-05-12 02:15:12Z petdance $
+ *     $Id: /mirror/trunk/include/parrot/pic.h 32171 2008-10-23T17:21:19.756490Z julianalbo  $
  *  Overview:
  *     This is the api header for the pic subsystem
  *  Data Structure and Algorithms:
@@ -82,7 +82,7 @@ PARROT_CANNOT_RETURN_NULL
 Parrot_PIC* parrot_PIC_alloc_pic(PARROT_INTERP)
         __attribute__nonnull__(1);
 
-void parrot_PIC_alloc_store(ARGOUT(struct PackFile_ByteCode *cs), size_t n)
+void parrot_PIC_alloc_store(ARGOUT(PackFile_ByteCode *cs), size_t n)
         __attribute__nonnull__(1)
         FUNC_MODIFIES(*cs);
 
@@ -96,7 +96,7 @@ int parrot_pic_check_sig(
         __attribute__nonnull__(3)
         FUNC_MODIFIES(*type);
 
-void parrot_PIC_destroy(ARGMOD(struct PackFile_ByteCode *cs))
+void parrot_PIC_destroy(ARGMOD(PackFile_ByteCode *cs))
         __attribute__nonnull__(1)
         FUNC_MODIFIES(*cs);
 
