@@ -1,7 +1,7 @@
 /* pmc.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: pmc.h 29253 2008-07-10 20:22:13Z julianalbo $
+ *     $Id: /mirror/trunk/include/parrot/pmc.h 32396 2008-11-05T21:09:46.188113Z chromatic  $
  *  Overview:
  *     This is the api header for the pmc subsystem
  *  Data Structure and Algorithms:
@@ -97,6 +97,13 @@ void dod_unregister_pmc(PARROT_INTERP, ARGIN(PMC* pmc))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CANNOT_RETURN_NULL
+PMC * temporary_pmc_new(PARROT_INTERP, INTVAL base_type)
+        __attribute__nonnull__(1);
+
+void temporary_pmc_free(PARROT_INTERP, PMC *pmc)
+        __attribute__nonnull__(1);
+
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 /* HEADERIZER END: src/pmc.c */
 
