@@ -1,7 +1,7 @@
 /* sub.h
  *  Copyright (C) 2001-2008, The Perl Foundation.
  *  SVN Info
- *     $Id: sub.h 32079 2008-10-21 20:35:06Z julianalbo $
+ *     $Id: /mirror/trunk/include/parrot/sub.h 32887 2008-11-19T02:30:14.715635Z chromatic  $
  *  Overview:
  *  Data Structure and Algorithms:
  *     Subroutine, coroutine, closure and continuation structures
@@ -39,7 +39,11 @@ typedef enum {
     /* [temporary expedient.  -- rgr, 13-Jul-08.] */
     SUB_FLAG_NEWCLOSURE   = SUB_FLAG_PF_IMMEDIATE,
 
-    SUB_FLAG_PF_MASK      = 0xfa   /* anon ... postcomp, is_outer*/
+    SUB_FLAG_PF_MASK      = SUB_FLAG_PF_ANON
+                          | SUB_FLAG_PF_MAIN
+                          | SUB_FLAG_PF_LOAD
+                          | SUB_FLAG_PF_IMMEDIATE
+                          | SUB_FLAG_PF_POSTCOMP,
 } sub_flags_enum;
 
 #define SUB_FLAG_get_FLAGS(o) (PObj_get_FLAGS(o))
@@ -138,7 +142,7 @@ typedef struct Parrot_sub_arginfo {
  * Closures have additionally an 'outer_ctx'
  */
 typedef struct Parrot_sub {
-    struct PackFile_ByteCode *seg;      /* bytecode segment */
+    PackFile_ByteCode *seg;     /* bytecode segment */
     size_t   start_offs;        /* sub entry in ops from seg->base.data */
     size_t   end_offs;
 
@@ -171,7 +175,7 @@ typedef struct Parrot_sub {
  * these two to the other type
  */
 typedef struct Parrot_coro {
-    struct PackFile_ByteCode *seg;      /* bytecode segment */
+    PackFile_ByteCode *seg;      /* bytecode segment */
     size_t   start_offs;         /* sub entry in ops from seg->base.data */
     size_t   end_offs;
 
@@ -196,7 +200,7 @@ typedef struct Parrot_coro {
 
     /* - end common */
 
-    struct PackFile_ByteCode *caller_seg;  /* bytecode segment */
+    PackFile_ByteCode *caller_seg;  /* bytecode segment */
     opcode_t *address;           /* next address to run - toggled each time */
     struct Stack_Chunk *dynamic_state; /* next dynamic state */
 } Parrot_coro;
@@ -205,7 +209,7 @@ typedef struct Parrot_coro {
 
 typedef struct Parrot_cont {
     /* continuation destination */
-    struct PackFile_ByteCode *seg;   /* bytecode segment */
+    PackFile_ByteCode *seg;          /* bytecode segment */
     opcode_t *address;               /* start of bytecode, addr to continue */
     struct Parrot_Context *to_ctx;   /* pointer to dest context */
     struct Stack_Chunk *dynamic_state; /* dest dynamic state */
