@@ -1,5 +1,5 @@
 # Copyright (C) 2007, The Perl Foundation.
-# $Id: Test.pm 31046 2008-09-13 00:03:19Z jonathan $
+# $Id: /mirror/trunk/languages/perl6/Test.pm 32134 2008-10-22T16:27:13.313862Z particle  $
 
 ## This is a temporary Test.pm to get us started until we get pugs's Test.pm
 ## working. It's shamelessly stolen & adapted from MiniPerl6 in the pugs repo.
@@ -24,121 +24,121 @@ sub approx ($x, $y) {
     ($diff < $epsilon);
 }
 
-sub plan($number_of_tests) {
+sub plan($number_of_tests) is export() {
     $testing_started      = 1;
     $num_of_tests_planned = $number_of_tests;
 
     say '1..' ~ $number_of_tests;
 }
 
-multi sub pass($desc) {
+multi sub pass($desc) is export() {
     proclaim(1, $desc);
 }
 
-multi sub ok(Object $cond, $desc) {
+multi sub ok(Object $cond, $desc) is export() {
     proclaim($cond, $desc);
 }
 
-multi sub ok(Object $cond) { ok($cond, ''); }
+multi sub ok(Object $cond) is export() { ok($cond, ''); }
 
 
-multi sub nok(Object $cond, $desc) {
+multi sub nok(Object $cond, $desc) is export() {
     proclaim(!$cond, $desc);
 }
 
-multi sub nok(Object $cond) { nok(!$cond, ''); }
+multi sub nok(Object $cond) is export() { nok(!$cond, ''); }
 
 
-multi sub is(Object $got, Object $expected, $desc) {
+multi sub is(Object $got, Object $expected, $desc) is export() {
     my $test = $got eq $expected;
     proclaim($test, $desc);
 }
 
-multi sub is(Object $got, Object $expected) { is($got, $expected, ''); }
+multi sub is(Object $got, Object $expected) is export() { is($got, $expected, ''); }
 
 
-multi sub isnt(Object $got, Object $expected, $desc) {
+multi sub isnt(Object $got, Object $expected, $desc) is export() {
     my $test = !($got eq $expected);
     proclaim($test, $desc);
 }
 
-multi sub isnt(Object $got, Object $expected) { isnt($got, $expected, ''); }
+multi sub isnt(Object $got, Object $expected) is export() { isnt($got, $expected, ''); }
 
-multi sub is_approx(Object $got, Object $expected, $desc) {
+multi sub is_approx(Object $got, Object $expected, $desc) is export() {
     my $test = abs($got - $expected) <= 0.00001;
     proclaim($test, $desc);
 }
 
-multi sub is_approx($got, $expected) { is_approx($got, $expected, ''); }
+multi sub is_approx($got, $expected) is export() { is_approx($got, $expected, ''); }
 
-multi sub todo($reason, $count) {
+multi sub todo($reason, $count) is export() {
     $todo_upto_test_num = $num_of_tests_run + $count;
     $todo_reason = '# TODO ' ~ $reason;
 }
 
-multi sub todo($reason) {
+multi sub todo($reason) is export() {
     $todo_upto_test_num = $num_of_tests_run + 1;
     $todo_reason = '# TODO ' ~ $reason;
 }
 
-multi sub skip()                { proclaim(1, "# SKIP"); }
-multi sub skip($reason)         { proclaim(1, "# SKIP " ~ $reason); }
-multi sub skip($count, $reason) {
+multi sub skip()                is export() { proclaim(1, "# SKIP"); }
+multi sub skip($reason)         is export() { proclaim(1, "# SKIP " ~ $reason); }
+multi sub skip($count, $reason) is export() {
     for 1..$count {
         proclaim(1, "# SKIP " ~ $reason);
     }
 }
 
-multi sub skip_rest() {
+multi sub skip_rest() is export() {
     skip($num_of_tests_planned - $num_of_tests_run, "");
 }
 
-multi sub skip_rest($reason) {
+multi sub skip_rest($reason) is export() {
     skip($num_of_tests_planned - $num_of_tests_run, $reason);
 }
 
-sub diag($message) { say '# '~$message; }
+sub diag($message) is export() { say '# '~$message; }
 
 
-multi sub flunk($reason) { proclaim(0, "flunk $reason")}
+multi sub flunk($reason) is export() { proclaim(0, "flunk $reason")}
 
 
-multi sub isa_ok($var,$type) {
+multi sub isa_ok($var,$type) is export() {
     ok($var.isa($type), "The object is-a '$type'");
 }
-multi sub isa_ok($var,$type, $msg) { ok($var.isa($type), $msg); }
+multi sub isa_ok($var,$type, $msg) is export() { ok($var.isa($type), $msg); }
 
-multi sub dies_ok($closure, $reason) {
+multi sub dies_ok($closure, $reason) is export() {
     try {
         $closure();
     }
     proclaim((defined $!), $reason);
 }
-multi sub dies_ok($closure) {
+multi sub dies_ok($closure) is export() {
     dies_ok($closure, '');
 }
 
-multi sub lives_ok($closure, $reason) {
+multi sub lives_ok($closure, $reason) is export() {
     try {
         $closure();
     }
     proclaim((not defined $!), $reason);
 }
-multi sub lives_ok($closure) {
+multi sub lives_ok($closure) is export() {
     lives_ok($closure, '');
 }
 
-multi sub eval_dies_ok($code, $reason) {
+multi sub eval_dies_ok($code, $reason) is export() {
     proclaim((defined eval_exception($code)), $reason);
 }
-multi sub eval_dies_ok($code) {
+multi sub eval_dies_ok($code) is export() {
     eval_dies_ok($code, '');
 }
 
-multi sub eval_lives_ok($code, $reason) {
+multi sub eval_lives_ok($code, $reason) is export() {
     proclaim((not defined eval_exception($code)), $reason);
 }
-multi sub eval_lives_ok($code) {
+multi sub eval_lives_ok($code) is export() {
     eval_lives_ok($code, '');
 }
 
