@@ -1,5 +1,5 @@
 # Copyright (C) 2008, The Perl Foundation.
-# $Id: actions.pm 31925 2008-10-13 11:47:49Z bernhard $
+# $Id: /mirror/trunk/languages/pipp/src/pct/actions.pm 32460 2008-11-07T17:07:30.582845Z bernhard  $
 
 =begin comments
 
@@ -15,6 +15,8 @@ value of the comment is passed as the second argument to the method.
 
 =end comments
 
+=cut
+
 class Pipp::Grammar::Actions;
 
 method TOP($/) {
@@ -42,7 +44,7 @@ method SEA($/) {
          );
 }
 
-method code_tp1($/) {
+method code_short_tag($/) {
     my $past := PAST::Stmts.new( :node($/) );
     for $<statement> {
         $past.push( $($_) );
@@ -51,7 +53,21 @@ method code_tp1($/) {
     make $past;
 }
 
-method code_tp2($/) {
+method code_echo_tag($/) {
+    my $past := PAST::Stmts.new( :node($/) );
+
+    my $echo := $( $<arguments> );
+    $echo.name( 'echo' );
+    $past.push( $echo );
+
+    for $<statement> {
+        $past.push( $($_) );
+    }
+
+    make $past;
+}
+
+method code_script_tag($/) {
     my $past := PAST::Stmts.new( :node($/) );
     for $<statement> {
         $past.push( $($_) );
@@ -73,7 +89,7 @@ method statement($/,$key) {
     make $( $/{$key} );
 }
 
-method inline_sea_tp1($/) {
+method inline_sea_short_tag($/) {
    make PAST::Op.new(
             PAST::Val.new(
                 :value(~$<SEA_empty_allowed>),
@@ -189,11 +205,12 @@ method var($/,$key) {
 }
 
 method VAR_NAME($/) {
+    our $?PIPP_SCOPE;
     make PAST::Var.new(
-             :scope('package'),
+             :scope( $?PIPP_SCOPE ?? $?PIPP_SCOPE !! 'package' ),
              :name(~$/),
              :viviself('Undef'),
-             :lvalue(1)
+             :lvalue(1),
          );
 }
 
@@ -203,6 +220,12 @@ method this($/) {
          );
 }
 
+method member($/) {
+    make PAST::Op.new(
+             :inline( "%r = self" )
+         );
+}
+
 method else_clause($/) {
     make $( $<block> );
 }
@@ -302,6 +325,9 @@ method NUMBER($/) {
 
 method function_definition($/) {
 
+    # PHP has two scopes: local to functions and global
+    our $?PIPP_SCOPE := 'lexical';
+
     # note that $<param_list> creates a new PAST::Block.
     my $past := $( $<param_list> );
 
@@ -309,6 +335,8 @@ method function_definition($/) {
     $past.control('return_pir');
     $past.push( $( $<block> ) );
 
+    $?PIPP_SCOPE := '';
+
     make $past;
 }
 
@@ -331,6 +359,7 @@ method param_list($/) {
                     :blocktype('declaration'),
                     :node($/)
                 );
+    my $arity := 0;
     for $<VAR_NAME> {
         my $param := $( $_ );
         $param.scope('parameter');
@@ -341,7 +370,9 @@ method param_list($/) {
              :scope('lexical'),
              $param.name()
         );
+        $arity++;
     }
+    $past.arity( $arity );
 
     make $past;
 }
