@@ -1,5 +1,5 @@
 # Copyright (C) 2006-2008, The Perl Foundation.
-# $Id: Harness.pm 31104 2008-09-14 09:54:06Z rurban $
+# $Id: /mirror/trunk/lib/Parrot/Test/Harness.pm 32130 2008-10-22T11:26:07.075376Z bernhard  $
 
 =head1 NAME
 
@@ -173,8 +173,9 @@ sub import {
               $options{exec}     ? ( exec => $options{exec} )
             : $options{compiler} ? ( exec => [ '../../parrot', './' . $options{compiler} ] )
             :                      ();
-        $harness_options{verbosity} = $options{verbosity} ? $options{verbosity} : 0;
-        $harness_options{jobs} = $ENV{TEST_JOBS} || $options{jobs} || 1;
+        $harness_options{verbosity} =                    $options{verbosity} || 0;
+        $harness_options{jobs}      = $ENV{TEST_JOBS} || $options{jobs}      || 1;
+
         TAP::Harness->new( \%harness_options )->runtests( @files );
 
         return;
