@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2008, The Perl Foundation.
-$Id: debug.c 32079 2008-10-21 20:35:06Z julianalbo $
+$Id: /mirror/trunk/src/debug.c 32480 2008-11-08T13:30:34.969342Z bernhard  $
 
 =head1 NAME
 
@@ -806,7 +806,7 @@ parse_command(ARGIN(const char *command), ARGOUT(unsigned long *cmdP))
 Debugger command line.
 
 Gets and executes commands, looping until the debugger state
-is chnaged, either to exit or to start executing code.
+is changed, either to exit or to start executing code.
 
 =cut
 
@@ -979,8 +979,11 @@ Parrot_debugger_start(PARROT_INTERP, ARGIN(opcode_t * cur_opcode))
 
     debugger_cmdline(interp);
 
-    if (interp->pdb->state & PDB_EXIT)
+    if (interp->pdb->state & PDB_EXIT) {
+        TRACEDEB_MSG("Parrot_debugger_start Parrot_exit");
         Parrot_exit(interp, 0);
+    }
+    TRACEDEB_MSG("Parrot_debugger_start ends");
 }
 
 /*
@@ -1959,6 +1962,8 @@ PDB_program_end(PARROT_INTERP)
 {
     PDB_t * const pdb = interp->pdb;
 
+    TRACEDEB_MSG("PDB_program_end");
+
     /* Remove the RUNNING state */
     pdb->state &= ~PDB_RUNNING;
 
@@ -3382,7 +3387,7 @@ static PDB_breakpoint_t *gdb_bps;
  * RT #46143 We can't remove the breakpoint yet, executing the next ins
  * most likely fails, as the length of the debug-brk stmt doesn't
  * match the old opcode
- * Setting a breakpoint will also fail, if the bytecode os r/o
+ * Setting a breakpoint will also fail, if the bytecode is r/o
  *
  */
 /*
