@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2008, The Perl Foundation.
-$Id: exceptions.c 32079 2008-10-21 20:35:06Z julianalbo $
+$Id: /mirror/trunk/src/exceptions.c 32844 2008-11-18T12:26:39.785982Z bernhard  $
 
 =head1 NAME
 
@@ -71,10 +71,8 @@ Parrot_ex_build_exception(PARROT_INTERP, INTVAL severity,
 {
     PMC *exception = pmc_new(interp, enum_class_Exception);
 
-    VTABLE_set_integer_keyed_str(interp, exception,
-            CONST_STRING(interp, "severity"), severity);
-    VTABLE_set_integer_keyed_str(interp, exception,
-            CONST_STRING(interp, "type"), error);
+    VTABLE_set_integer_keyed_str(interp, exception, CONST_STRING(interp, "severity"), severity);
+    VTABLE_set_integer_keyed_str(interp, exception, CONST_STRING(interp, "type"), error);
     if (msg)
         VTABLE_set_string_native(interp, exception, msg);
 
@@ -103,8 +101,7 @@ find_exception_handler(PARROT_INTERP, ARGIN(PMC *exception))
     else {
         STRING * const message = VTABLE_get_string(interp, exception);
         INTVAL exit_status = 1;
-        const INTVAL severity = VTABLE_get_integer_keyed_str(interp,
-                exception, CONST_STRING(interp, "severity"));
+        const INTVAL severity = VTABLE_get_integer_keyed_str(interp, exception, CONST_STRING(interp, "severity"));
 
         /* flush interpreter output to get things printed in order */
         PIO_flush(interp, PIO_STDOUT(interp));
@@ -122,8 +119,7 @@ find_exception_handler(PARROT_INTERP, ARGIN(PMC *exception))
         }
         else if (severity == EXCEPT_exit) {
             /* TODO: get exit status based on type */
-            exit_status = VTABLE_get_integer_keyed_str(interp,
-                    exception, CONST_STRING(interp, "exit_code"));
+            exit_status = VTABLE_get_integer_keyed_str(interp, exception, CONST_STRING(interp, "exit_code"));
         }
         else {
                 PIO_eprintf(interp, "No exception handler and no message\n");
@@ -138,9 +134,9 @@ find_exception_handler(PARROT_INTERP, ARGIN(PMC *exception))
          * RT #45917 this check should better be in Parrot_exit
          */
 
-        /* we should probably detach the thread here */
+        /* no exception handler, but this is not the main thread */
         if (interp->thread_data && interp->thread_data->tid)
-            return NULL;
+            pt_thread_detach(interp->thread_data->tid);
 
         /*
          * only main should run the destroy functions - exit handler chain
@@ -227,6 +223,19 @@ pass_exception_args(PARROT_INTERP, ARGIN(const char *sig),
     return next;
 }
 
+PARROT_CANNOT_RETURN_NULL
+static PMC *
+build_exception_from_args(PARROT_INTERP, int ex_type,
+        ARGIN(const char *format), va_list arglist)
+{
+    /* Make and set exception message. */
+    STRING *msg = strchr(format, '%') ?
+        Parrot_vsprintf_c(interp, format, arglist) :
+        string_make(interp, format, strlen(format), NULL, 0);
+
+    return Parrot_ex_build_exception(interp, EXCEPT_error, ex_type, msg);
+}
+
 /*
 
 =item C<void Parrot_ex_throw_from_c>
@@ -271,8 +280,7 @@ Parrot_ex_throw_from_c(PARROT_INTERP, ARGIN(PMC *exception))
     }
 
     if (Interp_debug_TEST(interp, PARROT_BACKTRACE_DEBUG_FLAG)) {
-        int exitcode = VTABLE_get_integer_keyed_str(interp, exception,
-                CONST_STRING(interp, "exit_code"));
+        int exitcode = VTABLE_get_integer_keyed_str(interp, exception, CONST_STRING(interp, "exit_code"));
         STRING *msg = VTABLE_get_string(interp, exception);
         PIO_eprintf(interp,
             "Parrot_ex_throw_from_c (severity:%d error:%d): %Ss\n",
@@ -298,6 +306,36 @@ Parrot_ex_throw_from_c(PARROT_INTERP, ARGIN(PMC *exception))
 
 /*
 
+=item C<opcode_t * Parrot_ex_throw_from_op_args>
+
+Throws an exception from an opcode, with an error message constructed
+from a format string and arguments.
+
+See also C<Parrot_ex_throw_from_c>, C<Parrot_ex_throw_from_op>, and
+C<exit_fatal()>.
+
+=cut
+
+*/
+
+PARROT_API
+PARROT_CAN_RETURN_NULL
+opcode_t *
+Parrot_ex_throw_from_op_args(PARROT_INTERP, ARGIN_NULLOK(void *dest),
+        int ex_type, ARGIN(const char *format), ...)
+{
+    PMC *exception;
+
+    va_list arglist;
+    va_start(arglist, format);
+    exception = build_exception_from_args(interp, ex_type, format, arglist);
+    va_end(arglist);
+
+    return Parrot_ex_throw_from_op(interp, exception, dest);
+}
+
+/*
+
 =item C<void Parrot_ex_throw_from_c_args>
 
 Throws an exception, with an error message constructed from a format string and
@@ -318,22 +356,12 @@ void
 Parrot_ex_throw_from_c_args(PARROT_INTERP, ARGIN_NULLOK(void *ret_addr),
         int exitcode, ARGIN(const char *format), ...)
 {
-    STRING *msg = NULL;
     PMC *exception;
 
-    /* Make and set exception message. */
-    if (strchr(format, '%')) {
-        va_list arglist;
-        va_start(arglist, format);
-        msg = Parrot_vsprintf_c(interp, format, arglist);
-        va_end(arglist);
-    }
-    else
-        msg = string_make(interp, format, strlen(format),
-                NULL, PObj_external_FLAG);
-
-    exception = Parrot_ex_build_exception(interp,
-            EXCEPT_error, exitcode, msg);
+    va_list arglist;
+    va_start(arglist, format);
+    exception = build_exception_from_args(interp, exitcode, format, arglist);
+    va_end(arglist);
 
     Parrot_ex_throw_from_c(interp, exception);
 }
@@ -397,8 +425,7 @@ PARROT_API
 void
 Parrot_ex_mark_unhandled(PARROT_INTERP, ARGIN(PMC *exception))
 {
-    VTABLE_set_integer_keyed_str(interp, exception,
-            CONST_STRING(interp, "handled"), -1);
+    VTABLE_set_integer_keyed_str(interp, exception, CONST_STRING(interp, "handled"), -1);
 }
 
 /*
