@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2008, The Perl Foundation.
-$Id: inter_call.c 32079 2008-10-21 20:35:06Z julianalbo $
+$Id: /mirror/trunk/src/inter_call.c 32844 2008-11-18T12:26:39.785982Z bernhard  $
 
 =head1 NAME
 
@@ -101,6 +101,7 @@ static void convert_arg_from_str(PARROT_INTERP, ARGMOD(call_state *st))
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*st);
 
+PARROT_CANNOT_RETURN_NULL
 static Parrot_Context * count_signature_elements(PARROT_INTERP,
     ARGIN(const char *signature),
     ARGMOD(PMC *args_sig),
@@ -145,6 +146,7 @@ static void null_val(int sig, ARGMOD(call_state *st))
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*st);
 
+PARROT_CAN_RETURN_NULL
 static const char * set_context_sig_params(PARROT_INTERP,
     ARGIN(const char *signature),
     ARGMOD(INTVAL *n_regs_used),
@@ -601,9 +603,16 @@ fetch_arg_op(PARROT_INTERP, ARGMOD(call_state *st))
             UVal_int(st->val) = constant ? idx : CTX_REG_INT(st->src.ctx, idx);
             break;
         case PARROT_ARG_STRING:
-            UVal_str(st->val) = constant ? st->src.ctx->constants[idx]->u.string
-                                         : CTX_REG_STR(st->src.ctx, idx);
+        {
+            /* ensure that callees don't modify constant caller strings */
+            if (constant)
+                UVal_str(st->val) = Parrot_make_COW_reference(interp,
+                                        st->src.ctx->constants[idx]->u.string);
+            else
+                UVal_str(st->val) = CTX_REG_STR(st->src.ctx, idx);
+
             break;
+        }
         case PARROT_ARG_FLOATVAL:
             UVal_num(st->val) = constant ? st->src.ctx->constants[idx]->u.number
                                          : CTX_REG_NUM(st->src.ctx, idx);
@@ -1934,6 +1943,7 @@ Adds the necessary registers to a new context and returns the context.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
 static Parrot_Context *
 count_signature_elements(PARROT_INTERP, ARGIN(const char *signature),
     ARGMOD(PMC *args_sig), ARGMOD(PMC *results_sig), int flag)
@@ -2227,6 +2237,7 @@ are the calls to C<commit_last_arg_sig_object> and C<commit_last_arg>.
 
 */
 
+PARROT_CAN_RETURN_NULL
 static const char *
 set_context_sig_params(PARROT_INTERP, ARGIN(const char *signature),
     ARGMOD(INTVAL *n_regs_used), ARGMOD(PMC **sigs),
@@ -2599,8 +2610,7 @@ Parrot_pcc_invoke_sub_from_sig_object(PARROT_INTERP, ARGIN(PMC *sub_obj),
     PMC * const args_sig    = pmc_new(interp, enum_class_FixedIntegerArray);
     PMC * const results_sig = pmc_new(interp, enum_class_FixedIntegerArray);
     PMC * const ret_cont    = new_ret_continuation_pmc(interp, NULL);
-    PMC * const result_list = VTABLE_get_attr_str(interp, sig_obj,
-            CONST_STRING(interp, "returns"));
+    PMC * const result_list = VTABLE_get_attr_str(interp, sig_obj, CONST_STRING(interp, "returns"));
 
     Parrot_Context *ctx;
     opcode_t         *dest;
@@ -2652,8 +2662,16 @@ Parrot_pcc_invoke_sub_from_sig_object(PARROT_INTERP, ARGIN(PMC *sub_obj),
 
     /* PIR Subs need runops to run their opcodes. */
     if (sub_obj->vtable->base_type == enum_class_Sub) {
-        opcode_t offset = dest - interp->code->base.data;
+        INTVAL old_core  = interp->run_core;
+        opcode_t offset  = dest - interp->code->base.data;
+
+        /* can't re-enter the runloop from here with PIC cores: RT #60048 */
+        if (interp->run_core == PARROT_CGP_CORE
+        ||  interp->run_core == PARROT_SWITCH_CORE)
+            interp->run_core = PARROT_SLOW_CORE;
+
         runops(interp, offset);
+        interp->run_core = old_core;
     }
 
     /* Set the return values from the subroutine's context into the
