@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2007-2008, The Perl Foundation.
-$Id: oo.c 32046 2008-10-20 05:00:46Z chromatic $
+$Id: /mirror/trunk/src/oo.c 32137 2008-10-22T17:00:55.080958Z bernhard  $
 
 =head1 NAME
 
@@ -401,28 +401,6 @@ Parrot_get_vtable_index(PARROT_INTERP, ARGIN(const STRING *name))
 }
 
 
-/*
-
-=item C<STRING* readable_name>
-
-Given a String or Key PMC return the STRING* representation
-
-{{{ DEPRECATED -- RT #45967 }}}
-
-=cut
-
-*/
-
-PARROT_API
-PARROT_WARN_UNUSED_RESULT
-PARROT_CANNOT_RETURN_NULL
-STRING*
-readable_name(PARROT_INTERP, ARGIN(PMC *name))
-{
-    return VTABLE_get_repr(interp, name);
-}
-
-
 /*
 
 =item C<const char* Parrot_MMD_method_name>
