@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2007-2008, The Perl Foundation.
-$Id: scheduler.c 31993 2008-10-16 05:54:03Z particle $
+$Id: /mirror/trunk/src/scheduler.c 32872 2008-11-19T00:25:47.089511Z chromatic  $
 
 =head1 NAME
 
@@ -72,7 +72,8 @@ Parrot_cx_init_scheduler(PARROT_INTERP)
         /* Add the very first interpreter to the list of interps. */
         pt_add_to_interpreters(interp, NULL);
 
-        scheduler = pmc_new(interp, enum_class_Scheduler);
+        /* create as constant to avoid order-of-destruction errors, RT #60622 */
+        scheduler = constant_pmc_new(interp, enum_class_Scheduler);
         scheduler = VTABLE_share_ro(interp, scheduler);
 
         interp->scheduler = scheduler;
@@ -483,7 +484,6 @@ void
 Parrot_cx_delete_handler_local(PARROT_INTERP, ARGIN(STRING *handler_type))
 {
     PMC *handlers  = CONTEXT(interp)->handlers;
-    INTVAL elements, index;
 
     if (PMC_IS_NULL(handlers))
         Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_INVALID_OPERATION,
@@ -491,27 +491,46 @@ Parrot_cx_delete_handler_local(PARROT_INTERP, ARGIN(STRING *handler_type))
 
     if (STRING_IS_NULL(handler_type) || STRING_IS_EMPTY(handler_type))
         VTABLE_shift_pmc(interp, handlers);
+    else {
+        /* Loop from newest handler to oldest handler. */
+        INTVAL index;
+        const INTVAL elements = VTABLE_elements(interp, handlers);
+        typedef enum { Hunknown,  Hexception, Hevent } Htype;
+        const Htype htype =
+            (string_equal(interp, handler_type, CONST_STRING(interp, "exception")) == 0) ?
+            Hexception :
+            (string_equal(interp, handler_type, CONST_STRING(interp, "event")) == 0) ?
+                Hevent :
+                Hunknown;
+        STRING * const handler_name = (htype == Hexception) ?
+            CONST_STRING(interp, "ExceptionHandler") :
+            (STRING *) NULL;
 
-    /* Loop from newest handler to oldest handler. */
-    elements = VTABLE_elements(interp, handlers);
-    for (index = 0; index < elements; ++index) {
-        PMC *handler = VTABLE_get_pmc_keyed_int(interp, handlers, index);
-        if (!PMC_IS_NULL(handler)) {
-            if (string_equal(interp, handler_type, CONST_STRING(interp, "exception")) == 0
-                        && handler->vtable->base_type == enum_class_ExceptionHandler) {
-                VTABLE_set_pmc_keyed_int(interp, handlers, index, PMCNULL);
-                return;
+        for (index = 0; index < elements; ++index) {
+            PMC *handler = VTABLE_get_pmc_keyed_int(interp, handlers, index);
+            if (!PMC_IS_NULL(handler)) {
+                switch (htype) {
+                    case Hexception:
+                        if (VTABLE_isa(interp, handler, handler_name)) {
+                            VTABLE_set_pmc_keyed_int(interp, handlers, index, PMCNULL);
+                            return;
+                        }
+                        break;
+                    case Hevent:
+                        if (handler->vtable->base_type == enum_class_EventHandler) {
+                            VTABLE_set_pmc_keyed_int(interp, handlers, index, PMCNULL);
+                            return;
+                        }
+                        break;
+                    default:
+                        break;
+                }
             }
-            else if (string_equal(interp, handler_type, CONST_STRING(interp, "event")) == 0
-                        && handler->vtable->base_type == enum_class_EventHandler) {
-                VTABLE_set_pmc_keyed_int(interp, handlers, index, PMCNULL);
-                return;
-           }
         }
-    }
 
-    Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_INVALID_OPERATION,
-        "No handler to delete.");
+        Parrot_ex_throw_from_c_args(interp, NULL,
+            EXCEPTION_INVALID_OPERATION, "No handler to delete.");
+    }
 }
 
 
@@ -727,10 +746,8 @@ Parrot_cx_find_handler_local(PARROT_INTERP, ARGIN(PMC *task))
     /* Exceptions store the handler iterator for rethrow, other kinds of
      * tasks don't (though they could). */
     if (task->vtable->base_type == enum_class_Exception &&
-            VTABLE_get_integer_keyed_str(interp, task,
-                CONST_STRING(interp, "handled")) == -1) {
-        iter    = VTABLE_get_attr_str(interp, task,
-                CONST_STRING(interp, "handler_iter"));
+            VTABLE_get_integer_keyed_str(interp, task, CONST_STRING(interp, "handled")) == -1) {
+        iter    = VTABLE_get_attr_str(interp, task, CONST_STRING(interp, "handler_iter"));
         context = (Parrot_Context *)VTABLE_get_pointer(interp, task);
     }
     else {
@@ -752,8 +769,7 @@ Parrot_cx_find_handler_local(PARROT_INTERP, ARGIN(PMC *task))
                 if (valid_handler) {
                     if (task->vtable->base_type == enum_class_Exception) {
                         /* Store iterator and context for a later rethrow. */
-                        VTABLE_set_attr_str(interp, task,
-                                CONST_STRING(interp, "handler_iter"), iter);
+                        VTABLE_set_attr_str(interp, task, CONST_STRING(interp, "handler_iter"), iter);
                         VTABLE_set_pointer(interp, task, context);
 
                         /* Mark that this handler has been used before. */
