@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2008, The Perl Foundation.
-$Id: string.c 31971 2008-10-15 07:13:36Z cotto $
+$Id: /mirror/trunk/src/string.c 32475 2008-11-08T03:44:37.823013Z chromatic  $
 
 =head1 NAME
 
@@ -2106,6 +2106,30 @@ string_to_cstring(PARROT_INTERP, ARGIN_NULLOK(const STRING *s))
         Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_UNEXPECTED_NULL,
             "Can't convert NULL string");
     }
+    else
+      return string_to_cstring_nullable(interp, s);
+}
+
+/*
+
+=item C<char * string_to_cstring_nullable>
+
+Returns a C string for the specified Parrot string. Use
+C<string_cstring_free()> to free the string. Failure to do this will result in
+a memory leak.
+
+=cut
+
+*/
+
+PARROT_API
+PARROT_MALLOC
+char *
+string_to_cstring_nullable(PARROT_INTERP, ARGIN_NULLOK(const STRING *s))
+{
+    if (! s) {
+      return NULL;
+    }
     else {
         char *p = (char *)mem_sys_allocate(s->bufused + 1);
         memcpy(p, s->strstart, s->bufused);
@@ -2370,8 +2394,8 @@ string_escape_string_delimited(PARROT_INTERP,
                     break;
             }
             if (c >= 0x20) {
-                dp[i++] = (unsigned char)c;
-                PARROT_ASSERT(i < charlen);
+                dp[i++]         = (unsigned char)c;
+                result->bufused = result->strlen = i;
                 continue;
             }
         }
