@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2008, The Perl Foundation.
-$Id: sub.c 32079 2008-10-21 20:35:06Z julianalbo $
+$Id: /mirror/trunk/src/sub.c 32627 2008-11-13T07:24:09.080607Z chromatic  $
 
 =head1 NAME
 
@@ -307,8 +307,33 @@ Parrot_full_sub_name(PARROT_INTERP, ARGIN_NULLOK(PMC* sub))
             STRING *j = CONST_STRING(interp, ";");
             STRING *res;
 
+            /*
+             * When running with -t4, the invoke done in
+             * Parrot_ns_get_name stomps on settings in interp; we
+             * have to save these and restore them to avoid affecting
+             * the running program.
+             */
+            PMC *saved_ccont            = interp->current_cont;
+            opcode_t *current_args      = interp->current_args;
+            opcode_t *current_params    = interp->current_params;
+            opcode_t *current_returns   = interp->current_returns;
+            PMC      *args_signature    = interp->args_signature;
+            PMC      *params_signature  = interp->params_signature;
+            PMC      *returns_signature = interp->returns_signature;
+
             Parrot_block_GC_mark(interp);
+
             ns_array = Parrot_ns_get_name(interp, s->namespace_stash);
+
+            /* Restore stuff that might have got overwritten */
+            interp->current_cont      = saved_ccont;
+            interp->current_args      = current_args;
+            interp->current_params    = current_params;
+            interp->current_returns   = current_returns;
+            interp->args_signature    = args_signature;
+            interp->params_signature  = params_signature;
+            interp->returns_signature = returns_signature;
+
             if (s->name)
                 VTABLE_push_string(interp, ns_array, s->name);
 
