@@ -1,7 +1,7 @@
 #! perl
 ################################################################################
 # Copyright (C) 2001-2003, The Perl Foundation.
-# $Id: cc_flags.pl 24274 2007-12-30 02:11:30Z chromatic $
+# $Id: /mirror/trunk/tools/dev/cc_flags.pl 32369 2008-11-04T18:01:31.882051Z tewk  $
 ################################################################################
 
 =head1 NAME
@@ -31,12 +31,18 @@ use strict;
 use warnings;
 
 my $return_only;
+my $verbose;
 
 if ($ARGV[0] eq '--return-only') {
     $return_only = 1;
     shift;
 }
 
+if ($ARGV[0] eq '-v') {
+    $verbose = 1;
+    shift;
+}
+
 my $cflags = shift;
 
 open my $F, '<', $cflags or die "open $cflags: $!\n";
@@ -139,6 +145,10 @@ if ($return_only) {
     exit;
 }
 
+if ($verbose) {
+    print join ' ', @ARGV;
+}
+
 exit system(@ARGV) / 256;
 
 # Local Variables:
